use actix_web::{error::ErrorInternalServerError, web, Responder};
use entity::node_group;
use sea_orm::{ActiveModelTrait, ActiveValue, EntityTrait, IntoActiveModel};
use serde::{Deserialize, Serialize};

use crate::AppState;

#[derive(Serialize)]
struct GroupWithNode {
    #[serde(flatten)]
    group: entity::node_group::Model,
    node: Vec<entity::node::Model>,
}

pub async fn get_nodes(state: web::Data<AppState>) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let result = node_group::Entity::find()
        .find_with_related(entity::prelude::Node)
        .all(db)
        .await
        .map_err(ErrorInternalServerError)?
        .into_iter()
        .map(|(group, node)| GroupWithNode { group, node })
        .collect::<Vec<_>>();

    Ok(web::Json(result))
}

pub async fn create_node(
    state: web::Data<AppState>,
    node: web::Json<entity::node::Model>,
) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let node = node.into_inner();

    let mut node = node.into_active_model();
    node.id = ActiveValue::NotSet;

    let result = node.insert(db).await.map_err(ErrorInternalServerError)?;

    Ok(web::Json(result))
}
