use sea_orm_migration::{prelude::*, schema::*};

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(NodeGroup::Table)
                    .if_not_exists()
                    .col(
                        uuid(NodeGroup::Id)
                            .extra("DEFAULT gen_random_uuid()")
                            .primary_key(),
                    )
                    .col(string(NodeGroup::Name))
                    .to_owned(),
            )
            .await?;

        manager
            .create_table(
                Table::create()
                    .table(Node::Table)
                    .if_not_exists()
                    .col(
                        uuid(Node::Id)
                            .extra("DEFAULT gen_random_uuid()")
                            .primary_key(),
                    )
                    .col(string(Node::Name))
                    .col(boolean(Node::Status))
                    .col(double(Node::CoordLa))
                    .col(double(Node::CoordLo))
                    .col(float(Node::Temperature))
                    .col(double(Node::Battery))
                    .col(big_unsigned(Node::Uptime))
                    .col(uuid(Node::Group))
                    .foreign_key(
                        ForeignKey::create()
                            .name("fk-node-group_id")
                            .from(Node::Table, Node::Group)
                            .to(NodeGroup::Table, NodeGroup::Id)
                            .on_update(ForeignKeyAction::Cascade)
                            .on_delete(ForeignKeyAction::Cascade),
                    )
                    .to_owned(),
            )
            .await?;

        Ok(())
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(Node::Table).to_owned())
            .await?;

        manager
            .drop_table(Table::drop().table(NodeGroup::Table).to_owned())
            .await?;

        Ok(())
    }
}

#[derive(DeriveIden)]
enum Node {
    Table,
    Id,
    Name, //Default mac address, kann auch geändert werden über die API
    Status,
    CoordLa,
    CoordLo,
    Temperature,
    Battery, //Measured in volts
    Uptime,
    Group,
}

#[derive(DeriveIden)]
enum NodeGroup {
    Table,
    Id,
    Name,
}
